Attribute VB_Name = "modUtil"
Option Explicit
Option Compare Binary
Option Base 0


' ShellExecute constants
' http://msdn.microsoft.com/library/en-us/shellcc/platform/Shell/reference/functions/shellexecute.asp
Public Enum ENUM_SHELLWINDOWSTYLE
     SW_HIDE = 0
     SW_SHOW = 5
     SW_DEFAULT = 10
     SW_MAXIMIZED = 3
     SW_MINIMIZED = 2
     SW_MINIMIZED_NOACTIVE = 7
     SW_NA = 8
     SW_NOACTIVE = 4
     SW_NORMAL = 1
End Enum

' ShellExecuteEx mask constants
Public Enum ENUM_SHELLEXECUTEMASK
     SEE_MASK_CLASSKEY = &H3
     SEE_MASK_CLASSNAME = &H1
     SEE_MASK_CONNECTNETDRV = &H80
     SEE_MASK_DOENVSUBST = &H200
     SEE_MASK_FLAG_DDEWAIT = &H100
     SEE_MASK_FLAG_NO_UI = &H400
     SEE_MASK_HOTKEY = &H20
     SEE_MASK_ICON = &H10
     SEE_MASK_IDLIST = &H4
     SEE_MASK_INVOKEIDLIST = &HC
     SEE_MASK_NOCLOSEPROCESS = &H40
End Enum


' Message constants
Private Const LB_ITEMFROMPOINT = &H1A9


' ShellExecuteEx structure
' http://msdn.microsoft.com/library/default.asp?url=/library/en-us/shellcc/platform/shell/reference/structures/shellexecuteinfo.asp
Private Type SHELLEXECUTEINFO
        cbSize As Long
        fMask As Long
        hWnd As Long
        lpVerb As String
        lpFile As String
        lpParameters As String
        lpDirectory As String
        nShow As Long
        hInstApp As Long
        
        ' Optional fields
        lpIDList As Long
        lpClass As String
        hkeyClass As Long
        dwHotKey As Long
        hIcon As Long
        hProcess As Long
End Type


' Win32 API calls
Public Declare Function ShellExecuteEx Lib "shell32.dll" Alias "ShellExecuteExA" (ByRef ExecInfo As SHELLEXECUTEINFO) As Boolean
Public Declare Function WaitForSingleObject Lib "kernel32.dll" (ByVal hHandle As Long, ByVal dwMilliseconds As Long) As Long
Public Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long
Private Declare Function GetLogicalDriveStrings Lib "kernel32" Alias "GetLogicalDriveStringsA" (ByVal nBufferLength As Long, ByVal lpBuffer As String) As Long
Private Declare Function GetVolumeInformation Lib "kernel32" Alias "GetVolumeInformationA" (ByVal lpRootPathName As String, ByVal lpVolumeNameBuffer As String, ByVal nVolumeNameSize As Long, lpVolumeSerialNumber As Long, lpMaximumComponentLength As Long, lpFileSystemFlags As Long, ByVal lpFileSystemNameBuffer As String, ByVal nFileSystemNameSize As Long) As Long


' Create a string of parameters out of an option list
Public Function ParameterString(ByVal cEngine As Integer) As String

    Dim A As Integer
    Dim Parameter As String
    
    
    For A = 0 To MaxOptions
    
        ' Only if the option is enabled of course...
        If Engines(cEngine).Options(A) = True Then
            ParameterString = ParameterString & " " & Engines(cEngine).Parameters(A).Parameter
        End If
    
    Next A
    
    ParameterString = LCase$(Trim$(ParameterString))

End Function


' Execute!
Public Function Execute(ByRef FileName As String, ByRef Parameters As String, ByVal WindowStyle As ENUM_SHELLWINDOWSTYLE, ByVal WaitForProcess As Boolean) As Boolean
     
     Dim ExecInfo As SHELLEXECUTEINFO
     
     
     ' Check if we should add local path
     If (InStr(FileName, "\") = 0) And _
        (InStr(FileName, "/") = 0) And _
        (InStr(FileName, ":") = 0) Then
          
          ' Add local path to filename
          FileName = App.Path & "\" & FileName
     End If
     
     ' Fill structure
     With ExecInfo
          .cbSize = Len(ExecInfo)
          .fMask = SEE_MASK_NOCLOSEPROCESS
          .lpFile = FileName
          .lpParameters = Parameters
          .lpDirectory = PathOf(FileName)
          .nShow = WindowStyle
     End With
     
      'Execute the file
     Execute = ShellExecuteEx(ExecInfo)
     
     ' Check if we should wait
     If (WaitForProcess) Then
          
          ' Wait for the process to end
          While WaitForSingleObject(ExecInfo.hProcess, 10): DoEvents: Wend
     End If
     
End Function


' Retrieve the path of a string
Public Function PathOf(ByVal FileName As String) As String
     
     On Error GoTo NoPath
     
     Dim FileTitle As String
     
     
     ' Get file title
     FileTitle = Dir(FileName)
     
     ' Return path only
     PathOf = Left$(FileName, Len(FileName) - Len(FileTitle))
     
     
NoPath:
     
     ' Leave now
     Exit Function
     
End Function


' Fill a combobox with map lump names from a WAD
Public Sub MapLumps(ByVal WadFile As String, ByRef cmbBox As ComboBox)

    Dim A As Long
    Dim File As New clsWad
    
    
    ' Open the WAD file
    File.OpenWad WadFile
    
    ' Scan through all lumps
    cmbBox.Clear
    For A = 0 To File.WadLumps - 1
    
        ' Add a new map lump
        If File.LumpName(A + 1) = "THINGS" Then
            
            ' Save the map into the string
            cmbBox.AddItem File.LumpName(A)
            
            ' Skip a few lumps that should belong to the map to speed things up
            A = A + 10
            
        End If
    
    Next A
    
    ' Destroy the WAD class
    Set File = Nothing

End Sub


' True = 1, False = 0
Public Function CCheck(ByVal InBool As Boolean) As Long

    If InBool = True Then CCheck = 1 Else CCheck = 0

End Function


' Create a paramater of a map lump's name
Public Function ParameterFromMapLump(ByVal LumpName As String) As String

    Dim Episode As Integer
    Dim Level As Integer
    
    
    ' Prepare
    LumpName = UCase$(LumpName)
    ParameterFromMapLump = "-warp "
    
    ' E*M* format (Doom\Heretic)
    If Len(LumpName) = 4 And Left$(LumpName, 1) = "E" And Mid$(LumpName, 3, 1) = "M" Then
        Episode = Val(Mid$(LumpName, 2, 1))
        Level = Val(Mid$(LumpName, 4, 1))
    
    ' MAP** format (Doom2\Hexen)
    ElseIf Len(LumpName) = 5 And Left$(LumpName, 3) = "MAP" Then
        Episode = -1
        Level = Val(Mid$(LumpName, 4, 2))
        
    ' Otherwise
    Else
        Episode = -2
    
    End If
    
    
    ' Add the map indicator
    If Episode = -1 Then
        ParameterFromMapLump = ParameterFromMapLump & Trim$(Str$(Level))
    
    ' Console warp. Not supported by vanilla of course
    ElseIf Episode = -2 Then
        ParameterFromMapLump = "+map " & LumpName
    
    ' E1M1
    Else
        ParameterFromMapLump = ParameterFromMapLump & Trim$(Str$(Episode)) & " " & Trim$(Str$(Level))
        
    End If

End Function


' Return the index of a listitem that the mouse cursor is hovering over
Public Function ListBoxHoverIndex(ByRef lstListBox As ListBox, ByVal X As Long, Y As Long) As Long

    Dim Returned As Long
        
    
    ' Convert coordinates to pixels
    X = X / Screen.TwipsPerPixelX
    Y = Y / Screen.TwipsPerPixelY
    
    ' Send the message
    Returned = SendMessage(lstListBox.hWnd, LB_ITEMFROMPOINT, 0, ByVal ((Y * 65536) + X))
    
    ' Analyse results
    If Returned < lstListBox.ListCount Then
        ListBoxHoverIndex = Returned
    Else
        ListBoxHoverIndex = -1
    End If

End Function


'  Return only the filename of a complete filename with path
Public Function StripPath(ByRef cString As String) As String

    StripPath = Mid$(cString, InStrRev(cString, "\") + 1)

End Function


' Return list of supported filetypes in Cdlg format
Public Function FileTypeList(ByVal cEngine As Long) As String

    Dim A As Long
    Dim SupportList As String
    
    
    ' Any useful engine?
    If cEngine < 0 Then
        FileTypeList = "WAD files (*.wad)|*.wad|All files (*.*)|*.*"
        Exit Function
    End If
    
    ' Generate list
    For A = 0 To UBound(Engines(cEngine).FileTypes())
        With Engines(cEngine).FileTypes(A)
            FileTypeList = FileTypeList & .Description & " (*." & .Extension & ")|*." & .Extension & "|"
            SupportList = SupportList & "*." & .Extension & ";"
        End With
    Next A
    
    ' Add last of types
    FileTypeList = FileTypeList & "All files (*.*)|*.*"
    FileTypeList = "All supported files|" & SupportList & "|" & FileTypeList

End Function


' Remove NULL characters from a string
Public Function StripNullChar(ByRef FixedString As String) As String
     
    Dim NullPos As Long
     
     
    ' Remove null terminator
    NullPos = InStr(1, FixedString, Chr$(0) & Chr$(0))
    If NullPos > 0 Then
        StripNullChar = Mid$(FixedString, 1, NullPos - 1)
    Else
        StripNullChar = FixedString
    End If
    
End Function


' parse a string of options into a boolean array
Public Sub ParseOptions(InString As String, ByRef BoolArray() As Boolean)

    Dim A As Long
    
    
    For A = 1 To Len(InString)
        If Mid$(InString, A, 1) = "1" Then BoolArray(A - 1) = True
    Next A

End Sub


' Return bitstring of options
Public Function OptionsString(Options() As Boolean) As String

    Dim A As Long
    
    
    For A = LBound(Options()) To UBound(Options())
        If Options(A) = True Then
            OptionsString = OptionsString & "1"
        Else
            OptionsString = OptionsString & "0"
        End If
    Next A

End Function


' Convert spaces to something else
Public Function SpaceConv(ByVal InString As String) As String

    SpaceConv = Replace$(InString, Chr$(32), Chr$(1))

End Function


' Remove the extension from a filename
Public Function RemoveExt(ByVal InString As String) As String

    If InStr(1, InString, ".") <> 0 Then
        RemoveExt = Left$(InString, InStrRev(InString, ".") - 1)
    Else
        RemoveExt = InString
    End If

End Function
